/*
     This file is part of GNUnet

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * Methods to access dynamic shared objects (DSOs).
 * @file server/dso.c
 * @author Christian Grothoff
 **/

#include "util/configuration.h"
#include "util/logging.h"
#include <dlfcn.h>
#include <errno.h>

static char * buildLibName(char * prefix,
			   char * dso) {
  char * dsoext;
  char * libname;

  dsoext = getConfigurationString("GNUNETD",
				  "DSOEXT");    
  if (dsoext == NULL) {
    dsoext = strdup(".so");
    LOG(LOG_WARNING,
	"WARNING: No DSOEXT defined in configuration, defaulting to %s\n",
	dsoext);
  }
  libname = xmalloc(strlen(dso) +
		    strlen(dsoext) +
		    strlen(prefix) + 1,
		    "buildLibName::libname");
  libname[0] = '\0';
  strcat(libname, prefix);
  strcat(libname, dso);
  strcat(libname, dsoext);
  xfree(dsoext,
	"dsoext");
  return libname;
}

void * loadDynamicLibrary(char * libprefix,
			  char * dsoname) {
  void * libhandle;
  char * libname;

  libname = buildLibName(libprefix,
			 dsoname);
  libhandle = dlopen(libname, RTLD_NOW); 
  if (libhandle == NULL)
    errexit("Could not open library %s (%s)!\n",
	    libname,
	    dlerror());
  xfree(libname, 
	"libname");
  return libhandle;
}

void unloadDynamicLibrary(void * libhandle) {
  dlclose(libhandle);
}

void * bindDynamicMethod(void * libhandle,
			 char * methodprefix,
			 char * dsoname) {
  char * initName;
  void * mptr;

  initName = xmalloc(strlen(dsoname) +
		     strlen(methodprefix) + 1,
		     "initName");
  initName[0] = '\0';
  strcat(initName, methodprefix);
  strcat(initName, dsoname);
  mptr = dlsym(libhandle, initName);
  if (mptr == NULL)
    errexit("Could not resolve method %s (%s)!\n",
	    initName,
	    dlerror());
  xfree(initName,
	"initName");  
  return mptr;
}

/* end of dso.c */			
